import { ApiProperty } from '@nestjs/swagger';
import {
  IsNotEmpty,
  IsString,
  IsArray,
  ValidateNested,
  IsOptional,
  IsNumber,
  IsBoolean,
} from 'class-validator';
import { Type } from 'class-transformer';

class EffectiveTime {
  @ApiProperty({ description: 'Start time', required: true })
  @IsString()
  @IsNotEmpty()
  public start: string;

  @ApiProperty({ description: 'End time', required: true })
  @IsString()
  @IsNotEmpty()
  public end: string;

  @ApiProperty({ description: 'Loops', required: true })
  @IsString()
  @IsNotEmpty()
  public loops: string;
}

class Expr {
  @ApiProperty({ description: 'Status code', required: true })
  @IsString()
  @IsNotEmpty()
  public statusCode: string;

  @ApiProperty({ description: 'Comparator', required: true })
  @IsString()
  @IsNotEmpty()
  public comparator: string;

  @ApiProperty({ description: 'Status value', required: true })
  @IsNotEmpty()
  public statusValue: any;
}

class Condition {
  @ApiProperty({ description: 'Condition code', required: true })
  @IsNumber()
  @IsNotEmpty()
  public code: number;

  @ApiProperty({ description: 'Entity ID', required: true })
  @IsString()
  @IsNotEmpty()
  public entityId: string;

  @ApiProperty({ description: 'Entity type', required: true })
  @IsString()
  @IsNotEmpty()
  public entityType: string;

  @ApiProperty({ description: 'Expression', required: true, type: Expr })
  @ValidateNested()
  @Type(() => Expr)
  @IsNotEmpty()
  public expr: Expr;
}

class ExecutorProperty {
  @ApiProperty({
    description: 'Function code (for device issue action)',
    required: false,
  })
  @IsString()
  @IsOptional()
  public functionCode?: string;

  @ApiProperty({
    description: 'Function value (for device issue action)',
    required: false,
  })
  @IsOptional()
  public functionValue?: any;

  @ApiProperty({
    description: 'Delay in seconds (for delay action)',
    required: false,
  })
  @IsNumber()
  @IsOptional()
  public delaySeconds?: number;
}

class Action {
  @ApiProperty({ description: 'Entity ID', required: true })
  @IsString()
  @IsNotEmpty()
  public entityId: string;

  @ApiProperty({ description: 'Action executor', required: true })
  @IsString()
  @IsNotEmpty()
  public actionExecutor: string;

  @ApiProperty({
    description: 'Executor property',
    required: false,
    type: ExecutorProperty,
  })
  @ValidateNested()
  @Type(() => ExecutorProperty)
  @IsOptional()
  public executorProperty?: ExecutorProperty;
}

export class AddAutomationDto {
  @ApiProperty({ description: 'Unit ID', required: true })
  @IsString()
  @IsNotEmpty()
  public unitUuid: string;

  @ApiProperty({ description: 'Automation name', required: true })
  @IsString()
  @IsNotEmpty()
  public automationName: string;

  @ApiProperty({ description: 'Decision expression', required: true })
  @IsString()
  @IsNotEmpty()
  public decisionExpr: string;

  @ApiProperty({
    description: 'Effective time',
    required: true,
    type: EffectiveTime,
  })
  @ValidateNested()
  @Type(() => EffectiveTime)
  @IsNotEmpty()
  public effectiveTime: EffectiveTime;

  @ApiProperty({ description: 'Conditions', required: true, type: [Condition] })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Condition)
  @IsNotEmpty()
  public conditions: Condition[];

  @ApiProperty({ description: 'Actions', required: true, type: [Action] })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Action)
  @IsNotEmpty()
  public actions: Action[];

  constructor(dto: Partial<AddAutomationDto>) {
    Object.assign(this, dto);
  }
}

export class UpdateAutomationDto {
  @ApiProperty({ description: 'Automation name', required: true })
  @IsString()
  @IsNotEmpty()
  public automationName: string;

  @ApiProperty({ description: 'Decision expression', required: true })
  @IsString()
  @IsNotEmpty()
  public decisionExpr: string;

  @ApiProperty({
    description: 'Effective time',
    required: true,
    type: EffectiveTime,
  })
  @ValidateNested()
  @Type(() => EffectiveTime)
  @IsNotEmpty()
  public effectiveTime: EffectiveTime;

  @ApiProperty({ description: 'Conditions', required: true, type: [Condition] })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Condition)
  @IsNotEmpty()
  public conditions: Condition[];

  @ApiProperty({ description: 'Actions', required: true, type: [Action] })
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => Action)
  @IsNotEmpty()
  public actions: Action[];
  constructor(dto: Partial<UpdateAutomationDto>) {
    Object.assign(this, dto);
  }
}
export class UpdateAutomationStatusDto {
  @ApiProperty({ description: 'Unit uuid', required: true })
  @IsString()
  @IsNotEmpty()
  public unitUuid: string;

  @ApiProperty({ description: 'Is enable', required: true })
  @IsBoolean()
  @IsNotEmpty()
  public isEnable: boolean;

  constructor(dto: Partial<UpdateAutomationStatusDto>) {
    Object.assign(this, dto);
  }
}
