import {
  Injectable,
  HttpException,
  HttpStatus,
  BadRequestException,
} from '@nestjs/common';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import {
  AddAutomationDto,
  UpdateAutomationDto,
  UpdateAutomationStatusDto,
} from '../dtos';
import { GetUnitByUuidInterface } from 'src/unit/interface/unit.interface';
import { ConfigService } from '@nestjs/config';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { convertKeysToSnakeCase } from '@app/common/helper/snakeCaseConverter';
import { DeviceService } from 'src/device/services';
import {
  AddAutomationInterface,
  AutomationDetailsResult,
  AutomationResponseData,
  DeleteAutomationInterface,
  GetAutomationByUnitInterface,
} from '../interface/automation.interface';
import { convertKeysToCamelCase } from '@app/common/helper/camelCaseConverter';

@Injectable()
export class AutomationService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly spaceRepository: SpaceRepository,
    private readonly deviceService: DeviceService,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }

  async addAutomation(addAutomationDto: AddAutomationDto, spaceTuyaId = null) {
    try {
      let unitSpaceTuyaId;
      if (!spaceTuyaId) {
        const unitDetails = await this.getUnitByUuid(addAutomationDto.unitUuid);

        unitSpaceTuyaId = unitDetails.spaceTuyaUuid;
        if (!unitDetails) {
          throw new BadRequestException('Invalid unit UUID');
        }
      } else {
        unitSpaceTuyaId = spaceTuyaId;
      }

      const actions = addAutomationDto.actions.map((action) =>
        convertKeysToSnakeCase(action),
      );
      const conditions = addAutomationDto.conditions.map((condition) =>
        convertKeysToSnakeCase(condition),
      );

      for (const action of actions) {
        if (action.action_executor === 'device_issue') {
          const device = await this.deviceService.getDeviceByDeviceUuid(
            action.entity_id,
            false,
          );
          if (device) {
            action.entity_id = device.deviceTuyaUuid;
          }
        }
      }

      for (const condition of conditions) {
        if (condition.entity_type === 'device_report') {
          const device = await this.deviceService.getDeviceByDeviceUuid(
            condition.entity_id,
            false,
          );
          if (device) {
            condition.entity_id = device.deviceTuyaUuid;
          }
        }
      }

      const path = `/v2.0/cloud/scene/rule`;
      const response: AddAutomationInterface = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          space_id: unitSpaceTuyaId,
          name: addAutomationDto.automationName,
          effective_time: {
            ...addAutomationDto.effectiveTime,
            timezone_id: 'Asia/Dubai',
          },
          type: 'automation',
          decision_expr: addAutomationDto.decisionExpr,
          conditions: conditions,
          actions: actions,
        },
      });
      if (!response.success) {
        throw new HttpException(response.msg, HttpStatus.BAD_REQUEST);
      }
      return {
        id: response.result.id,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          err.message || 'Automation not found',
          err.status || HttpStatus.NOT_FOUND,
        );
      }
    }
  }
  async getUnitByUuid(unitUuid: string): Promise<GetUnitByUuidInterface> {
    try {
      const unit = await this.spaceRepository.findOne({
        where: {
          uuid: unitUuid,
          spaceType: {
            type: 'unit',
          },
        },
        relations: ['spaceType'],
      });
      if (!unit || !unit.spaceType || unit.spaceType.type !== 'unit') {
        throw new BadRequestException('Invalid unit UUID');
      }
      return {
        uuid: unit.uuid,
        createdAt: unit.createdAt,
        updatedAt: unit.updatedAt,
        name: unit.spaceName,
        type: unit.spaceType.type,
        spaceTuyaUuid: unit.spaceTuyaUuid,
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Unit not found', HttpStatus.NOT_FOUND);
      }
    }
  }
  async getAutomationByUnit(unitUuid: string) {
    try {
      const unit = await this.getUnitByUuid(unitUuid);
      if (!unit.spaceTuyaUuid) {
        throw new BadRequestException('Invalid unit UUID');
      }

      const path = `/v2.0/cloud/scene/rule?space_id=${unit.spaceTuyaUuid}&type=automation`;
      const response: GetAutomationByUnitInterface = await this.tuya.request({
        method: 'GET',
        path,
      });

      if (!response.success) {
        throw new HttpException(response.msg, HttpStatus.BAD_REQUEST);
      }

      return response.result.list.map((item) => {
        return {
          id: item.id,
          name: item.name,
          status: item.status,
          type: 'automation',
        };
      });
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          err.message || 'Automation not found',
          err.status || HttpStatus.NOT_FOUND,
        );
      }
    }
  }
  async getTapToRunSceneDetailsTuya(
    sceneId: string,
  ): Promise<AutomationDetailsResult> {
    try {
      const path = `/v2.0/cloud/scene/rule/${sceneId}`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      if (!response.success) {
        throw new HttpException(response.msg, HttpStatus.BAD_REQUEST);
      }
      const camelCaseResponse = convertKeysToCamelCase(response);
      const { id, name, type } = camelCaseResponse.result;

      return {
        id,
        name,
        type,
      } as AutomationDetailsResult;
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          'Scene not found for Tuya',
          HttpStatus.NOT_FOUND,
        );
      }
    }
  }
  async getAutomationDetails(automationId: string, withSpaceId = false) {
    try {
      const path = `/v2.0/cloud/scene/rule/${automationId}`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      if (!response.success) {
        throw new HttpException(response.msg, HttpStatus.BAD_REQUEST);
      }

      const responseData: AutomationResponseData = convertKeysToCamelCase(
        response.result,
      );

      const actions = responseData.actions.map((action) => ({
        ...action,
      }));

      for (const action of actions) {
        if (action.actionExecutor === 'device_issue') {
          const device = await this.deviceService.getDeviceByDeviceTuyaUuid(
            action.entityId,
          );

          if (device) {
            action.entityId = device.uuid;
          }
        } else if (
          action.actionExecutor !== 'device_issue' &&
          action.actionExecutor !== 'delay'
        ) {
          const sceneDetails = await this.getTapToRunSceneDetailsTuya(
            action.entityId,
          );

          if (sceneDetails.id) {
            action.name = sceneDetails.name;
            action.type = sceneDetails.type;
          }
        }
      }

      const conditions = responseData.conditions.map((condition) => ({
        ...condition,
      }));

      for (const condition of conditions) {
        if (condition.entityType === 'device_report') {
          const device = await this.deviceService.getDeviceByDeviceTuyaUuid(
            condition.entityId,
          );

          if (device) {
            condition.entityId = device.uuid;
          }
        }
      }

      // eslint-disable-next-line @typescript-eslint/no-unused-vars
      const { timeZoneId, ...effectiveTimeWithoutTimeZoneId } =
        responseData.effectiveTime || {};

      return {
        id: responseData.id,
        name: responseData.name,
        status: responseData.status,
        type: 'automation',
        ...(() => {
          // eslint-disable-next-line @typescript-eslint/no-unused-vars
          const { spaceId, runningMode, ...rest } = responseData;
          return rest;
        })(),
        actions,
        conditions,
        effectiveTime: effectiveTimeWithoutTimeZoneId, // Use modified effectiveTime
        ...(withSpaceId && { spaceId: responseData.spaceId }),
      };
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException('Automation not found', HttpStatus.NOT_FOUND);
      }
    }
  }

  async deleteAutomation(
    unitUuid: string,
    automationId: string,
    spaceTuyaId = null,
  ) {
    try {
      let unitSpaceTuyaId;
      if (!spaceTuyaId) {
        const unitDetails = await this.getUnitByUuid(unitUuid);
        unitSpaceTuyaId = unitDetails.spaceTuyaUuid;
        if (!unitSpaceTuyaId) {
          throw new BadRequestException('Invalid unit UUID');
        }
      } else {
        unitSpaceTuyaId = spaceTuyaId;
      }

      const path = `/v2.0/cloud/scene/rule?ids=${automationId}&space_id=${unitSpaceTuyaId}`;
      const response: DeleteAutomationInterface = await this.tuya.request({
        method: 'DELETE',
        path,
      });

      if (!response.success) {
        throw new HttpException('Automation not found', HttpStatus.NOT_FOUND);
      }

      return response;
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          err.message || 'Automation not found',
          err.status || HttpStatus.NOT_FOUND,
        );
      }
    }
  }

  async updateAutomation(
    updateAutomationDto: UpdateAutomationDto,
    automationId: string,
  ) {
    try {
      const spaceTuyaId = await this.getAutomationDetails(automationId, true);
      if (!spaceTuyaId.spaceId) {
        throw new HttpException(
          "Automation doesn't exist",
          HttpStatus.NOT_FOUND,
        );
      }
      const addAutomation = {
        ...updateAutomationDto,
        unitUuid: null,
      };
      const newAutomation = await this.addAutomation(
        addAutomation,
        spaceTuyaId.spaceId,
      );
      if (newAutomation.id) {
        await this.deleteAutomation(null, automationId, spaceTuyaId.spaceId);
        return newAutomation;
      }
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          err.message || 'Automation not found',
          err.status || HttpStatus.NOT_FOUND,
        );
      }
    }
  }
  async updateAutomationStatus(
    updateAutomationStatusDto: UpdateAutomationStatusDto,
    automationId: string,
  ) {
    try {
      const unitDetails = await this.getUnitByUuid(
        updateAutomationStatusDto.unitUuid,
      );
      if (!unitDetails.spaceTuyaUuid) {
        throw new BadRequestException('Invalid unit UUID');
      }

      const path = `/v2.0/cloud/scene/rule/state?space_id=${unitDetails.spaceTuyaUuid}`;
      const response: DeleteAutomationInterface = await this.tuya.request({
        method: 'PUT',
        path,
        body: {
          ids: automationId,
          is_enable: updateAutomationStatusDto.isEnable,
        },
      });

      if (!response.success) {
        throw new HttpException('Automation not found', HttpStatus.NOT_FOUND);
      }

      return response;
    } catch (err) {
      if (err instanceof BadRequestException) {
        throw err; // Re-throw BadRequestException
      } else {
        throw new HttpException(
          err.message || 'Automation not found',
          err.status || HttpStatus.NOT_FOUND,
        );
      }
    }
  }
}
