import { ProductRepository } from './../../../libs/common/src/modules/product/repositories/product.repository';
import {
  Injectable,
  HttpException,
  HttpStatus,
  NotFoundException,
  BadRequestException,
} from '@nestjs/common';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { ConfigService } from '@nestjs/config';
import { AddDeviceDto, UpdateDeviceInRoomDto } from '../dtos/add.device.dto';
import {
  DeviceInstructionResponse,
  GetDeviceDetailsFunctionsInterface,
  GetDeviceDetailsFunctionsStatusInterface,
  GetDeviceDetailsInterface,
  controlDeviceInterface,
  updateDeviceFirmwareInterface,
} from '../interfaces/get.device.interface';
import { GetDeviceByRoomUuidDto } from '../dtos/get.device.dto';
import { ControlDeviceDto } from '../dtos/control.device.dto';
import { convertKeysToCamelCase } from '@app/common/helper/camelCaseConverter';
import { DeviceRepository } from '@app/common/modules/device/repositories';
import { PermissionType } from '@app/common/constants/permission-type.enum';
import { In } from 'typeorm';
import { ProductType } from '@app/common/constants/product-type.enum';
import { SpaceRepository } from '@app/common/modules/space/repositories';

@Injectable()
export class DeviceService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly deviceRepository: DeviceRepository,
    private readonly productRepository: ProductRepository,
    private readonly spaceRepository: SpaceRepository,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }
  async getDeviceByDeviceUuid(
    deviceUuid: string,
    withProductDevice: boolean = true,
  ) {
    return await this.deviceRepository.findOne({
      where: {
        uuid: deviceUuid,
      },
      ...(withProductDevice && { relations: ['productDevice'] }),
    });
  }
  async getDeviceByDeviceTuyaUuid(deviceTuyaUuid: string) {
    return await this.deviceRepository.findOne({
      where: {
        deviceTuyaUuid,
      },
      relations: ['productDevice'],
    });
  }
  async addDeviceUser(addDeviceDto: AddDeviceDto) {
    try {
      const device = await this.getDeviceDetailsByDeviceIdTuya(
        addDeviceDto.deviceTuyaUuid,
      );

      if (!device.productUuid) {
        throw new Error('Product UUID is missing for the device.');
      }

      return await this.deviceRepository.save({
        deviceTuyaUuid: addDeviceDto.deviceTuyaUuid,
        productDevice: { uuid: device.productUuid },
        user: {
          uuid: addDeviceDto.userUuid,
        },
      });
    } catch (error) {
      if (error.code === '23505') {
        throw new HttpException(
          'Device already exists',
          HttpStatus.BAD_REQUEST,
        );
      } else {
        throw new HttpException(
          error.message || 'Failed to add device in room',
          error.status || HttpStatus.INTERNAL_SERVER_ERROR,
        );
      }
    }
  }
  async getDevicesByUser(
    userUuid: string,
  ): Promise<GetDeviceDetailsInterface[]> {
    try {
      const devices = await this.deviceRepository.find({
        where: {
          user: { uuid: userUuid },
          permission: {
            userUuid,
            permissionType: {
              type: In([PermissionType.READ, PermissionType.CONTROLLABLE]),
            },
          },
        },
        relations: [
          'spaceDevice',
          'productDevice',
          'permission',
          'permission.permissionType',
        ],
      });
      const devicesData = await Promise.all(
        devices.map(async (device) => {
          return {
            haveRoom: device.spaceDevice ? true : false,
            productUuid: device.productDevice.uuid,
            productType: device.productDevice.prodType,
            permissionType: device.permission[0].permissionType.type,
            ...(await this.getDeviceDetailsByDeviceIdTuya(
              device.deviceTuyaUuid,
            )),
            uuid: device.uuid,
          } as GetDeviceDetailsInterface;
        }),
      );

      return devicesData;
    } catch (error) {
      // Handle the error here
      throw new HttpException(
        'User does not have any devices',
        HttpStatus.NOT_FOUND,
      );
    }
  }
  async getDevicesByRoomId(
    getDeviceByRoomUuidDto: GetDeviceByRoomUuidDto,
    userUuid: string,
  ): Promise<GetDeviceDetailsInterface[]> {
    try {
      const devices = await this.deviceRepository.find({
        where: {
          spaceDevice: { uuid: getDeviceByRoomUuidDto.roomUuid },
          permission: {
            userUuid,
            permissionType: {
              type: In([PermissionType.READ, PermissionType.CONTROLLABLE]),
            },
          },
        },
        relations: [
          'spaceDevice',
          'productDevice',
          'permission',
          'permission.permissionType',
        ],
      });
      const devicesData = await Promise.all(
        devices.map(async (device) => {
          return {
            haveRoom: device.spaceDevice ? true : false,
            productUuid: device.productDevice.uuid,
            productType: device.productDevice.prodType,
            permissionType: device.permission[0].permissionType.type,
            ...(await this.getDeviceDetailsByDeviceIdTuya(
              device.deviceTuyaUuid,
            )),
            uuid: device.uuid,
          } as GetDeviceDetailsInterface;
        }),
      );

      return devicesData;
    } catch (error) {
      // Handle the error here
      throw new HttpException(
        'Error fetching devices by room',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async updateDeviceInRoom(updateDeviceInRoomDto: UpdateDeviceInRoomDto) {
    try {
      await this.deviceRepository.update(
        { uuid: updateDeviceInRoomDto.deviceUuid },
        {
          spaceDevice: { uuid: updateDeviceInRoomDto.roomUuid },
        },
      );
      const device = await this.deviceRepository.findOne({
        where: {
          uuid: updateDeviceInRoomDto.deviceUuid,
        },
        relations: ['spaceDevice', 'spaceDevice.parent'],
      });
      if (device.spaceDevice.parent.spaceTuyaUuid) {
        await this.transferDeviceInSpacesTuya(
          device.deviceTuyaUuid,
          device.spaceDevice.parent.spaceTuyaUuid,
        );
      }

      return {
        uuid: device.uuid,
        roomUuid: device.spaceDevice.uuid,
      };
    } catch (error) {
      throw new HttpException(
        'Failed to add device in room',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async transferDeviceInSpacesTuya(
    deviceId: string,
    spaceId: string,
  ): Promise<controlDeviceInterface> {
    try {
      const path = `/v2.0/cloud/thing/${deviceId}/transfer`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: { space_id: spaceId },
      });

      return response as controlDeviceInterface;
    } catch (error) {
      throw new HttpException(
        'Error transferring device in spaces from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async controlDevice(controlDeviceDto: ControlDeviceDto, deviceUuid: string) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(deviceUuid, false);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new NotFoundException('Device Not Found');
      }
      const response = await this.controlDeviceTuya(
        deviceDetails.deviceTuyaUuid,
        controlDeviceDto,
      );

      if (response.success) {
        return response;
      } else {
        throw new HttpException(
          response.msg || 'Unknown error',
          HttpStatus.BAD_REQUEST,
        );
      }
    } catch (error) {
      throw new HttpException(
        error.message || 'Device Not Found',
        error.status || HttpStatus.NOT_FOUND,
      );
    }
  }
  async controlDeviceTuya(
    deviceUuid: string,
    controlDeviceDto: ControlDeviceDto,
  ): Promise<controlDeviceInterface> {
    try {
      const path = `/v1.0/iot-03/devices/${deviceUuid}/commands`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
        body: {
          commands: [
            { code: controlDeviceDto.code, value: controlDeviceDto.value },
          ],
        },
      });

      return response as controlDeviceInterface;
    } catch (error) {
      throw new HttpException(
        'Error control device from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getDeviceDetailsByDeviceId(deviceUuid: string, userUuid: string) {
    try {
      const userDevicePermission = await this.getUserDevicePermission(
        userUuid,
        deviceUuid,
      );

      const deviceDetails = await this.getDeviceByDeviceUuid(deviceUuid);

      if (!deviceDetails) {
        throw new NotFoundException('Device Not Found');
      }

      const response = await this.getDeviceDetailsByDeviceIdTuya(
        deviceDetails.deviceTuyaUuid,
      );

      return {
        ...response,
        uuid: deviceDetails.uuid,
        productUuid: deviceDetails.productDevice.uuid,
        productType: deviceDetails.productDevice.prodType,
        permissionType: userDevicePermission,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Device Not Found',
        HttpStatus.NOT_FOUND,
      );
    }
  }
  async getDeviceDetailsByDeviceIdTuya(
    deviceId: string,
  ): Promise<GetDeviceDetailsInterface> {
    try {
      const path = `/v1.1/iot-03/devices/${deviceId}`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      // Convert keys to camel case
      const camelCaseResponse = convertKeysToCamelCase(response);
      const product = await this.productRepository.findOne({
        where: {
          prodId: camelCaseResponse.result.productId,
        },
      });

      // eslint-disable-next-line @typescript-eslint/no-unused-vars
      const { productName, productId, id, ...rest } = camelCaseResponse.result;

      return {
        ...rest,
        productUuid: product.uuid,
      } as GetDeviceDetailsInterface;
    } catch (error) {
      throw new HttpException(
        'Error fetching device details from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getDeviceInstructionByDeviceId(
    deviceUuid: string,
  ): Promise<DeviceInstructionResponse> {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(deviceUuid);

      if (!deviceDetails) {
        throw new NotFoundException('Device Not Found');
      }

      const response = await this.getDeviceInstructionByDeviceIdTuya(
        deviceDetails.deviceTuyaUuid,
      );

      return {
        productUuid: deviceDetails.productDevice.uuid,
        productType: deviceDetails.productDevice.prodType,
        functions: response.result.functions.map((fun: any) => {
          return {
            code: fun.code,
            values: fun.values,
            dataType: fun.type,
          };
        }),
      };
    } catch (error) {
      throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
    }
  }

  async getDeviceInstructionByDeviceIdTuya(
    deviceId: string,
  ): Promise<GetDeviceDetailsFunctionsInterface> {
    try {
      const path = `/v1.0/iot-03/devices/${deviceId}/functions`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
      });
      return response as GetDeviceDetailsFunctionsInterface;
    } catch (error) {
      throw new HttpException(
        'Error fetching device functions from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getDevicesInstructionStatus(deviceUuid: string) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(deviceUuid);

      if (!deviceDetails) {
        throw new NotFoundException('Device Not Found');
      }
      const deviceStatus = await this.getDevicesInstructionStatusTuya(
        deviceDetails.deviceTuyaUuid,
      );

      return {
        productUuid: deviceDetails.productDevice.uuid,
        productType: deviceDetails.productDevice.prodType,
        status: deviceStatus.result[0].status,
      };
    } catch (error) {
      throw new HttpException(
        'Error fetching device functions status',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getDevicesInstructionStatusTuya(
    deviceUuid: string,
  ): Promise<GetDeviceDetailsFunctionsStatusInterface> {
    try {
      const path = `/v1.0/iot-03/devices/status`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
        query: {
          device_ids: deviceUuid,
        },
      });
      return response as GetDeviceDetailsFunctionsStatusInterface;
    } catch (error) {
      throw new HttpException(
        'Error fetching device functions status from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  private async getUserDevicePermission(userUuid: string, deviceUuid: string) {
    const device = await this.deviceRepository.findOne({
      where: {
        uuid: deviceUuid,
        permission: {
          userUuid: userUuid,
        },
      },
      relations: ['permission', 'permission.permissionType'],
    });
    return device.permission[0].permissionType.type;
  }
  async getDevicesInGateway(gatewayUuid: string) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(gatewayUuid);

      if (!deviceDetails) {
        throw new NotFoundException('Device Not Found');
      } else if (deviceDetails.productDevice.prodType !== ProductType.GW) {
        throw new BadRequestException('This is not a gateway device');
      }

      const response = await this.getDevicesInGatewayTuya(
        deviceDetails.deviceTuyaUuid,
      );

      const devices = await Promise.all(
        response.map(async (device: any) => {
          const deviceDetails = await this.getDeviceByDeviceTuyaUuid(device.id);
          if (deviceDetails.deviceTuyaUuid) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            const { id, ...rest } = device;
            return {
              ...rest,
              tuyaUuid: deviceDetails.deviceTuyaUuid,
              uuid: deviceDetails.uuid,
              productUuid: deviceDetails.productDevice.uuid,
              productType: deviceDetails.productDevice.prodType,
            };
          }
          return null;
        }),
      );

      return {
        uuid: deviceDetails.uuid,
        productUuid: deviceDetails.productDevice.uuid,
        productType: deviceDetails.productDevice.prodType,
        devices: devices.filter((device) => device !== null),
      };
    } catch (error) {
      throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
    }
  }

  async getDevicesInGatewayTuya(
    deviceId: string,
  ): Promise<GetDeviceDetailsInterface[]> {
    try {
      const path = `/v1.0/devices/${deviceId}/sub-devices`;
      const response: any = await this.tuya.request({
        method: 'GET',
        path,
      });
      const camelCaseResponse = response.result.map((device: any) => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const { product_id, category, ...rest } = device;
        const camelCaseDevice = convertKeysToCamelCase({ ...rest });
        return camelCaseDevice as GetDeviceDetailsInterface[];
      });

      return camelCaseResponse;
    } catch (error) {
      throw new HttpException(
        'Error fetching device details from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async updateDeviceFirmware(deviceUuid: string, firmwareVersion: number) {
    try {
      const deviceDetails = await this.getDeviceByDeviceUuid(deviceUuid, false);

      if (!deviceDetails || !deviceDetails.deviceTuyaUuid) {
        throw new NotFoundException('Device Not Found');
      }
      const response = await this.updateDeviceFirmwareTuya(
        deviceDetails.deviceTuyaUuid,
        firmwareVersion,
      );

      if (response.success) {
        return response;
      } else {
        throw new HttpException(
          response.msg || 'Unknown error',
          HttpStatus.BAD_REQUEST,
        );
      }
    } catch (error) {
      throw new HttpException('Device Not Found', HttpStatus.NOT_FOUND);
    }
  }
  async updateDeviceFirmwareTuya(
    deviceUuid: string,
    firmwareVersion: number,
  ): Promise<updateDeviceFirmwareInterface> {
    try {
      const path = `/v2.0/cloud/thing/${deviceUuid}/firmware/${firmwareVersion}`;
      const response = await this.tuya.request({
        method: 'POST',
        path,
      });

      return response as updateDeviceFirmwareInterface;
    } catch (error) {
      throw new HttpException(
        'Error updating device firmware from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  async getDevicesByUnitId(unitUuid: string) {
    try {
      const spaces = await this.spaceRepository.find({
        where: {
          parent: {
            uuid: unitUuid,
          },
        },
        relations: ['devicesSpaceEntity', 'devicesSpaceEntity.productDevice'],
      });

      const devices = spaces.flatMap((space) => {
        return space.devicesSpaceEntity.map((device) => device);
      });

      const devicesData = await Promise.all(
        devices.map(async (device) => {
          return {
            haveRoom: true,
            productUuid: device.productDevice.uuid,
            productType: device.productDevice.prodType,
            permissionType: PermissionType.CONTROLLABLE,
            ...(await this.getDeviceDetailsByDeviceIdTuya(
              device.deviceTuyaUuid,
            )),
            uuid: device.uuid,
          } as GetDeviceDetailsInterface;
        }),
      );

      return devicesData;
    } catch (error) {
      throw new HttpException(
        'This unit does not have any devices',
        HttpStatus.NOT_FOUND,
      );
    }
  }
}
