import { FloorService } from '../services/floor.service';
import {
  Body,
  Controller,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Post,
  Put,
  Query,
  UseGuards,
} from '@nestjs/common';
import { ApiTags, ApiBearerAuth } from '@nestjs/swagger';
import { AddFloorDto, AddUserFloorDto } from '../dtos/add.floor.dto';
import { GetFloorChildDto } from '../dtos/get.floor.dto';
import { UpdateFloorNameDto } from '../dtos/update.floor.dto';
import { CheckBuildingTypeGuard } from 'src/guards/building.type.guard';
import { CheckUserFloorGuard } from 'src/guards/user.floor.guard';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import { AdminRoleGuard } from 'src/guards/admin.role.guard';
import { FloorPermissionGuard } from 'src/guards/floor.permission.guard';

@ApiTags('Floor Module')
@Controller({
  version: '1',
  path: 'floor',
})
export class FloorController {
  constructor(private readonly floorService: FloorService) {}

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, CheckBuildingTypeGuard)
  @Post()
  async addFloor(@Body() addFloorDto: AddFloorDto) {
    try {
      const floor = await this.floorService.addFloor(addFloorDto);
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'Floor added successfully',
        data: floor,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, FloorPermissionGuard)
  @Get(':floorUuid')
  async getFloorByUuid(@Param('floorUuid') floorUuid: string) {
    try {
      const floor = await this.floorService.getFloorByUuid(floorUuid);
      return floor;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, FloorPermissionGuard)
  @Get('child/:floorUuid')
  async getFloorChildByUuid(
    @Param('floorUuid') floorUuid: string,
    @Query() query: GetFloorChildDto,
  ) {
    try {
      const floor = await this.floorService.getFloorChildByUuid(
        floorUuid,
        query,
      );
      return floor;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, FloorPermissionGuard)
  @Get('parent/:floorUuid')
  async getFloorParentByUuid(@Param('floorUuid') floorUuid: string) {
    try {
      const floor = await this.floorService.getFloorParentByUuid(floorUuid);
      return floor;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard, CheckUserFloorGuard)
  @Post('user')
  async addUserFloor(@Body() addUserFloorDto: AddUserFloorDto) {
    try {
      await this.floorService.addUserFloor(addUserFloorDto);
      return {
        statusCode: HttpStatus.CREATED,
        success: true,
        message: 'user floor added successfully',
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get('user/:userUuid')
  async getFloorsByUserId(@Param('userUuid') userUuid: string) {
    try {
      return await this.floorService.getFloorsByUserId(userUuid);
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard, FloorPermissionGuard)
  @Put('rename/:floorUuid')
  async renameFloorByUuid(
    @Param('floorUuid') floorUuid: string,
    @Body() updateFloorNameDto: UpdateFloorNameDto,
  ) {
    try {
      const floor = await this.floorService.renameFloorByUuid(
        floorUuid,
        updateFloorNameDto,
      );
      return floor;
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
