import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { TuyaContext } from '@tuya/tuya-connector-nodejs';
import { ConfigService } from '@nestjs/config';
import { SpaceRepository } from '@app/common/modules/space/repositories';
import { GetDeviceDetailsInterface } from 'src/device/interfaces/get.device.interface';
import { convertKeysToCamelCase } from '@app/common/helper/camelCaseConverter';
import { ProductRepository } from '@app/common/modules/product/repositories';
import { PermissionType } from '@app/common/constants/permission-type.enum';
import { In } from 'typeorm';

@Injectable()
export class GroupService {
  private tuya: TuyaContext;
  constructor(
    private readonly configService: ConfigService,
    private readonly productRepository: ProductRepository,
    private readonly spaceRepository: SpaceRepository,
  ) {
    const accessKey = this.configService.get<string>('auth-config.ACCESS_KEY');
    const secretKey = this.configService.get<string>('auth-config.SECRET_KEY');
    const tuyaEuUrl = this.configService.get<string>('tuya-config.TUYA_EU_URL');
    this.tuya = new TuyaContext({
      baseUrl: tuyaEuUrl,
      accessKey,
      secretKey,
    });
  }

  async getGroupsByUnitUuid(unitUuid: string) {
    try {
      const spaces = await this.spaceRepository.find({
        where: {
          parent: {
            uuid: unitUuid,
          },
        },
        relations: ['devicesSpaceEntity', 'devicesSpaceEntity.productDevice'],
      });

      const groupNames = spaces.flatMap((space) => {
        return space.devicesSpaceEntity.map(
          (device) => device.productDevice.prodType,
        );
      });

      const uniqueGroupNames = [...new Set(groupNames)];

      return uniqueGroupNames.map((groupName) => ({ groupName }));
    } catch (error) {
      throw new HttpException(
        'This unit does not have any groups',
        HttpStatus.NOT_FOUND,
      );
    }
  }

  async getUnitDevicesByGroupName(
    unitUuid: string,
    groupName: string,
    userUuid: string,
  ) {
    try {
      const spaces = await this.spaceRepository.find({
        where: {
          parent: {
            uuid: unitUuid,
          },
          devicesSpaceEntity: {
            productDevice: {
              prodType: groupName,
            },
            permission: {
              userUuid,
              permissionType: {
                type: In([PermissionType.READ, PermissionType.CONTROLLABLE]),
              },
            },
          },
        },
        relations: [
          'devicesSpaceEntity',
          'devicesSpaceEntity.productDevice',
          'devicesSpaceEntity.spaceDevice',
          'devicesSpaceEntity.permission',
          'devicesSpaceEntity.permission.permissionType',
        ],
      });

      const devices = await Promise.all(
        spaces.flatMap(async (space) => {
          return await Promise.all(
            space.devicesSpaceEntity.map(async (device) => {
              const deviceDetails = await this.getDeviceDetailsByDeviceIdTuya(
                device.deviceTuyaUuid,
              );
              return {
                haveRoom: !!device.spaceDevice,
                productUuid: device.productDevice.uuid,
                productType: device.productDevice.prodType,
                permissionType: device.permission[0]?.permissionType?.type,
                ...deviceDetails,
                uuid: device.uuid,
              };
            }),
          );
        }),
      );
      if (devices.length === 0)
        throw new HttpException('No devices found', HttpStatus.NOT_FOUND);
      return devices.flat(); // Flatten the array since flatMap was used
    } catch (error) {
      throw new HttpException(
        'This unit does not have any devices for the specified group name',
        HttpStatus.NOT_FOUND,
      );
    }
  }

  async getDeviceDetailsByDeviceIdTuya(
    deviceId: string,
  ): Promise<GetDeviceDetailsInterface> {
    try {
      const path = `/v1.1/iot-03/devices/${deviceId}`;
      const response = await this.tuya.request({
        method: 'GET',
        path,
      });

      // Convert keys to camel case
      const camelCaseResponse = convertKeysToCamelCase(response);
      const product = await this.productRepository.findOne({
        where: {
          prodId: camelCaseResponse.result.productId,
        },
      });

      // eslint-disable-next-line @typescript-eslint/no-unused-vars
      const { productName, productId, ...rest } = camelCaseResponse.result;

      return {
        ...rest,
        productUuid: product.uuid,
      } as GetDeviceDetailsInterface;
    } catch (error) {
      throw new HttpException(
        'Error fetching device details from Tuya',
        HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
