import { SpaceRepository } from '@app/common/modules/space/repositories';
import {
  Injectable,
  CanActivate,
  HttpStatus,
  BadRequestException,
  ExecutionContext,
} from '@nestjs/common';

@Injectable()
export class CheckBuildingTypeGuard implements CanActivate {
  constructor(private readonly spaceRepository: SpaceRepository) {}

  async canActivate(context: ExecutionContext): Promise<boolean> {
    const req = context.switchToHttp().getRequest();

    try {
      const { floorName, buildingUuid } = req.body;

      if (!floorName) {
        throw new BadRequestException('floorName is required');
      }

      if (!buildingUuid) {
        throw new BadRequestException('buildingUuid is required');
      }

      await this.checkBuildingIsBuildingType(buildingUuid);

      return true;
    } catch (error) {
      this.handleGuardError(error, context);
      return false;
    }
  }

  async checkBuildingIsBuildingType(buildingUuid: string) {
    const buildingData = await this.spaceRepository.findOne({
      where: { uuid: buildingUuid },
      relations: ['spaceType'],
    });
    if (
      !buildingData ||
      !buildingData.spaceType ||
      buildingData.spaceType.type !== 'building'
    ) {
      throw new BadRequestException('Invalid building UUID');
    }
  }

  private handleGuardError(error: Error, context: ExecutionContext) {
    const response = context.switchToHttp().getResponse();
    console.error(error);

    if (error instanceof BadRequestException) {
      response
        .status(HttpStatus.BAD_REQUEST)
        .json({ statusCode: HttpStatus.BAD_REQUEST, message: error.message });
    } else {
      response.status(HttpStatus.NOT_FOUND).json({
        statusCode: HttpStatus.NOT_FOUND,
        message: 'Building not found',
      });
    }
  }
}
