import {
  Body,
  Controller,
  Delete,
  Get,
  HttpException,
  HttpStatus,
  Param,
  Post,
  Put,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiTags } from '@nestjs/swagger';
import { UserDevicePermissionService } from '../services/user-device-permission.service';
import { UserDevicePermissionAddDto } from '../dtos/user-device-permission.add.dto';
import { UserDevicePermissionEditDto } from '../dtos/user-device-permission.edit.dto';
import { AdminRoleGuard } from 'src/guards/admin.role.guard';

@ApiTags('Device Permission Module')
@Controller({
  version: '1',
  path: 'device-permission',
})
export class UserDevicePermissionController {
  constructor(
    private readonly userDevicePermissionService: UserDevicePermissionService,
  ) {}

  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard)
  @Post('add')
  async addDevicePermission(
    @Body() userDevicePermissionDto: UserDevicePermissionAddDto,
  ) {
    try {
      const addDetails =
        await this.userDevicePermissionService.addUserPermission(
          userDevicePermissionDto,
        );
      return {
        statusCode: HttpStatus.CREATED,
        message: 'User Permission for Devices Added Successfully',
        data: addDetails,
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard)
  @Put('edit/:devicePermissionUuid')
  async editDevicePermission(
    @Param('devicePermissionUuid') devicePermissionUuid: string,
    @Body() userDevicePermissionEditDto: UserDevicePermissionEditDto,
  ) {
    try {
      await this.userDevicePermissionService.editUserPermission(
        devicePermissionUuid,
        userDevicePermissionEditDto,
      );
      return {
        statusCode: HttpStatus.OK,
        message: 'User Permission for Devices Updated Successfully',
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard)
  @Get(':deviceUuid/list')
  async fetchDevicePermission(@Param('deviceUuid') deviceUuid: string) {
    try {
      const deviceDetails =
        await this.userDevicePermissionService.fetchUserPermission(deviceUuid);
      return {
        statusCode: HttpStatus.OK,
        message: 'Device Details fetched Successfully',
        data: deviceDetails,
      };
    } catch (err) {
      throw new Error(err);
    }
  }
  @ApiBearerAuth()
  @UseGuards(AdminRoleGuard)
  @Delete(':devicePermissionUuid')
  async deleteDevicePermission(
    @Param('devicePermissionUuid') devicePermissionUuid: string,
  ) {
    try {
      await this.userDevicePermissionService.deleteDevicePermission(
        devicePermissionUuid,
      );
      return {
        statusCode: HttpStatus.OK,
        message: 'User Permission for Devices Deleted Successfully',
      };
    } catch (error) {
      throw new HttpException(
        error.message || 'Internal server error',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }
}
